// du_dxddug.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2019 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file was created on Thu  8 Jun 2017 15:38:06 CEST by ./convertMG5.sh
// VINCIA C++ wrapper for du_dxddug.f

#ifndef VINCIA_du_dxddug_H
#define VINCIA_du_dxddug_H

#include "Vincia/MG5interface.h"

namespace Vincia {
using namespace Pythia8;

class MG5du_dxddug : public MG5matrix {

 public:

  // Constructor
  MG5du_dxddug() { initHelMap(); initColMap(); };

  // Destructor
  virtual ~MG5du_dxddug() {};

  // Name
  virtual string name() {return "du_dxddug";};

  // PDG ID multilabel
  virtual vector< vector<int> > processList();

  // Number of legs
  virtual int nLegs() {return 7;}

  // Redefine methods to define/initialise helicity and colour maps
  virtual void initHelMap();
  virtual void initColMap();

  // Store momenta in MG5 format (from input in Pythia Vec4 format)
  // (Must be copied in derived classes so that pSav will be the
  // variable in the derived class, rather than the one in the base class)
  virtual void setMom(vector<Particle> pIn, bool permute=true) {
    for (int i=0; i<nLegs(); ++i) {
      if (permute) {
        pSav[i][0] = pIn[iVincia[i]].e();
        pSav[i][1] = pIn[iVincia[i]].px();
        pSav[i][2] = pIn[iVincia[i]].py();
        pSav[i][3] = pIn[iVincia[i]].pz();
      } else {
        pSav[i][0] = pIn[i].e();
        pSav[i][1] = pIn[i].px();
        pSav[i][2] = pIn[i].py();
        pSav[i][3] = pIn[i].pz();
      }
    }
  }

  // setMom with vector<Vec4> as input (hardcoded for speed)
  virtual void setMom(vector<Vec4> pIn, bool permute=true) {
    for (int i=0; i<nLegs(); ++i) {
      if (permute) {
        pSav[i][0] = pIn[iVincia[i]].e();
  	pSav[i][1] = pIn[iVincia[i]].px();
        pSav[i][2] = pIn[iVincia[i]].py();
        pSav[i][3] = pIn[iVincia[i]].pz();
      } else {
        pSav[i][0] = pIn[i].e();
        pSav[i][1] = pIn[i].px();
        pSav[i][2] = pIn[i].py();
        pSav[i][3] = pIn[i].pz();
      }
    }
  }

  // Print momenta (in MG enumeration)
  virtual void printMom() {
    for (int i=0; i<nLegs(); ++i) {
      cout<<" pMG"<<i<<setprecision(6)<<" = ";
      for (int j = 0; j<4; ++j) cout<<" "<<setw(16)<<pSav[i][j];
      cout<<endl;
    }
  }

  // Wrapper for MG5 matrix element squared
  virtual double getME2(int mcMode);

 protected: 

  // Momenta in MadGraph format: p[NLEGS][4]
  double pSav[7][4];

}; // end class MG5du_dxddug

} // end namespace Vincia

#endif

