// dxd_dxd.cc is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2019 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users,
//               http://www.montecarlonet.org/GUIDELINES

// This file was created on Thu  8 Jun 2017 15:37:53 CEST by ./convertMG5.sh
// VINCIA C++ wrapper for dxd_dxd.f

#include "VinciaMG5/dxd_dxd.h"

namespace Vincia {

//====================================================================

// Wrapper for the MG5 FORTRAN subroutine itself

extern "C" {
  extern void sheldxd_dxd_(int *cMode, int *ICOL, double P1[4][4], int *HEL, double* ANS);
}

//--------------------------------------------------------------------

// initHelMap(): create map from vector of helicities to IHEL number

void MG5dxd_dxd::initHelMap() {
  // Hardcoded helicity map extracted from .f file
  vector<int> h; h.resize(4);
  h[0]=-1;  h[1]=1;  h[2]=1;  h[3]=-1;  hMapSav[h] = 1;
  h[0]=-1;  h[1]=1;  h[2]=1;  h[3]=1;  hMapSav[h] = 2;
  h[0]=-1;  h[1]=1;  h[2]=-1;  h[3]=-1;  hMapSav[h] = 3;
  h[0]=-1;  h[1]=1;  h[2]=-1;  h[3]=1;  hMapSav[h] = 4;
  h[0]=-1;  h[1]=-1;  h[2]=1;  h[3]=-1;  hMapSav[h] = 5;
  h[0]=-1;  h[1]=-1;  h[2]=1;  h[3]=1;  hMapSav[h] = 6;
  h[0]=-1;  h[1]=-1;  h[2]=-1;  h[3]=-1;  hMapSav[h] = 7;
  h[0]=-1;  h[1]=-1;  h[2]=-1;  h[3]=1;  hMapSav[h] = 8;
  h[0]=1;  h[1]=1;  h[2]=1;  h[3]=-1;  hMapSav[h] = 9;
  h[0]=1;  h[1]=1;  h[2]=1;  h[3]=1;  hMapSav[h] = 10;
  h[0]=1;  h[1]=1;  h[2]=-1;  h[3]=-1;  hMapSav[h] = 11;
  h[0]=1;  h[1]=1;  h[2]=-1;  h[3]=1;  hMapSav[h] = 12;
  h[0]=1;  h[1]=-1;  h[2]=1;  h[3]=-1;  hMapSav[h] = 13;
  h[0]=1;  h[1]=-1;  h[2]=1;  h[3]=1;  hMapSav[h] = 14;
  h[0]=1;  h[1]=-1;  h[2]=-1;  h[3]=-1;  hMapSav[h] = 15;
  h[0]=1;  h[1]=-1;  h[2]=-1;  h[3]=1;  hMapSav[h] = 16;
}

//--------------------------------------------------------------------

// initColMap(): create map from vector of MG indices to ICOL numbers

void MG5dxd_dxd::initColMap() {
  // Hardcoded colour-index map extracted from .f file
  // Colour traces extracted from MG5 fortran file
  vector< vector<int> > traces; traces.resize(0);
  vector<int> traceNow;
  traceNow.resize(0);
  // ICOL 1
  traceNow.push_back(1);
  traceNow.push_back(2);
  traces.push_back(traceNow);
  traceNow.resize(0);
  traceNow.push_back(4);
  traceNow.push_back(3);
  traces.push_back(traceNow);
  // Traces should appear ordered by lowest first index
  if (traces[1][0] < traces[0][0]) {
    vector<int> tmp = traces[0];
    traces[0] = traces[1];
    traces[1] = tmp;
  }
  // Put elements of traces[] into traceNow in correct order
  traceNow=traces[0];
  for (int i=1; i<(int)traces.size(); ++i)
    traceNow.insert(traceNow.end(),traces[i].begin(),traces[i].end());
  traces.resize(0);
  cMapSav[traceNow]=1;
  traceNow.resize(0);
  // ICOL 2
  traceNow.push_back(1);
  traceNow.push_back(3);
  traces.push_back(traceNow);
  traceNow.resize(0);
  traceNow.push_back(4);
  traceNow.push_back(2);
  traces.push_back(traceNow);
  // Traces should appear ordered by lowest first index
  if (traces[1][0] < traces[0][0]) {
    vector<int> tmp = traces[0];
    traces[0] = traces[1];
    traces[1] = tmp;
  }
  // Put elements of traces[] into traceNow in correct order
  traceNow=traces[0];
  for (int i=1; i<(int)traces.size(); ++i)
    traceNow.insert(traceNow.end(),traces[i].begin(),traces[i].end());
  traces.resize(0);
  cMapSav[traceNow]=2;
}

//------------------------------------------------------------------

// Return vector of all process labels matched by this ME

vector< vector<int> > MG5dxd_dxd::processList() {
  vector< vector<int> > labels; labels.resize(0);
  vector<int> thisLabel;
  thisLabel.resize(0);
  thisLabel.push_back(-1);
  thisLabel.push_back(1);
  thisLabel.push_back(-1);
  thisLabel.push_back(1);
  labels.push_back(thisLabel);
  thisLabel.resize(0);
  thisLabel.push_back(-2);
  thisLabel.push_back(2);
  thisLabel.push_back(-2);
  thisLabel.push_back(2);
  labels.push_back(thisLabel);
  thisLabel.resize(0);
  thisLabel.push_back(-3);
  thisLabel.push_back(3);
  thisLabel.push_back(-3);
  thisLabel.push_back(3);
  labels.push_back(thisLabel);
  thisLabel.resize(0);
  thisLabel.push_back(-4);
  thisLabel.push_back(4);
  thisLabel.push_back(-4);
  thisLabel.push_back(4);
  labels.push_back(thisLabel);
  // Sanity check
  for (int i=0; i<(int)labels.size(); ++i) {
    if (nLegs() != (int)labels[i].size())
      cout<<"  MG5${procName}::processList(): ERROR! inconsistent number of legs"<<endl;
  }
  return labels;
}

//------------------------------------------------------------------

// Wrapper for MG5 ME, with variables set by the other methods

double MG5dxd_dxd::getME2(int mcMode) {
  // Call wrapped matrix element
  double ans;
  sheldxd_dxd_(&mcMode, &icolSav, pSav, &ihelSav, &ans);
  return ans;
}

} // End namespace Vincia

